local abs,max,min,floor,ceil = math.abs,math.max,math.min,math.floor,math.ceil
function math.sign(n) return n>0 and 1 or n<0 and -1 or 0 end
lg = love.graphics
lt = love.timer
lk = love.keyboard

width,height = lg.getDimensions()
lg.setDefaultFilter("nearest","nearest")
lg.setLineWidth(2)
lg.setBackgroundColor(0,50,50)

ts = 32

lume = require "lib.lume"
hc = require "lib.HC"

function love.load()
	map = {
	{0,0,0,0,5},
	{0,0,0,0,4},
	{0,0,0,0,3},
	{0,0,0,0,2},
	{0,0,0,0,1},
	}
	local vx,vy = 8,128+32 --map offset
	for y=1,#map do for x=1,#map[y] do -- go through map, give each cell a collision and height value
		local tx,ty = ts*(x-1)+ts*(y-1)+vx,-ts/2*x + ts/2*(y-1)+vy
		map[y][x] = {h=map[y][x], col=hc.polygon(tx,ty, tx+ts,ty+ts/2, tx+ts*2,ty, tx+ts,ty-ts/2)}
	end end
	
	plyr = {x=64,y=128,xv=0,yv=0, z=0,zv=0, img=lg.newImage("plyr.png")}
	local t,maxi = {},32
	for i=1,maxi,2 do
		t[i]  = 18*math.cos((i/maxi)*2*math.pi)+plyr.x
		t[i+1]= 6*math.sin((i/maxi)*2*math.pi)+plyr.y
	end
	plyr.col = hc.polygon(unpack(t))
	pAnm = {f=1,t=0}; for i=1,4 do pAnm[i] = lg.newImage("plyr/plyr"..i..".png") end
	iTile = lg.newImage("tile.png")
end

local minh = 0
function love.update(dt)
	love.window.setTitle("spurp - FPS:"..lt.getFPS())
	-- determine if the player is colliding with an actual cell
	-- if so, then determine if the player's z value is >= the cell height value (or 0 if undefined)
	-- if true, don't push away, else push away
	local p = plyr
	local moving = false
	local spd = 1720
	if lk.isDown("right") then p.xv = p.xv+spd*dt; moving=true end
	if lk.isDown("left")  then p.xv = p.xv-spd*dt; moving=true end
	if lk.isDown("up")    then p.yv = p.yv-spd*dt; moving=true end
	if lk.isDown("down")  then p.yv = p.yv+spd*dt; moving=true end
	
	-- smooth moves
	p.x = lume.clamp(p.x,0,400) + p.xv * dt
	p.y = lume.clamp(p.y,0,240) + p.yv * dt
	p.xv = p.xv * (1-dt*10)
	p.yv = p.yv * (1-dt*10)
	p.col:moveTo(p.x,p.y)
	
	plyr.zv = plyr.zv + 12*dt
	if p.z+0.5 > minh then p.zv = minh end -- change the 0 to a check on which tile the plyr is on and its height
	if p.z > 0 then p.z = 0 end -- not really good, maybe redo?
	if lk.isDown("space") and min(p.z,0) >= minh then p.zv = -5 end -- if landed, jump
	p.z = p.z + p.zv * (1 - 12*dt)
	minh = 0
	
	for y=1,#map do for x=1,#map[y] do
		local collides,dx,dy = plyr.col:collidesWith(map[x][y].col)
		if collides then
			if abs(p.z) >= map[x][y].h*ts then minh = map[x][y].h*ts
			else p.x,p.y = p.x+dx,p.y+dy; p.col:moveTo(p.x,p.y)
		end end
	end end
	
	if moving then
		pAnm.t = pAnm.t + dt
		while pAnm.t >= 0.08 do
			pAnm.t = pAnm.t - 0.08
			pAnm.f = pAnm.f + 1
			if pAnm.f > 4 then pAnm.f = 1 end
		end
	end
end

function print_r ( t )  
    local print_r_cache={}
    local function sub_print_r(t,indent)
        if (print_r_cache[tostring(t)]) then
            print(indent.."*"..tostring(t))
        else
            print_r_cache[tostring(t)]=true
            if (type(t)=="table") then
                for pos,val in pairs(t) do
                    if (type(val)=="table") then
                        print(indent.."["..pos.."] => "..tostring(t).." {")
                        sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
                        print(indent..string.rep(" ",string.len(pos)+6).."}")
                    elseif (type(val)=="string") then
                        print(indent.."["..pos..'] => "'..val..'"')
                    else
                        print(indent.."["..pos.."] => "..tostring(val))
                    end
                end
            else
                print(indent..tostring(t))
            end
        end
    end
    if (type(t)=="table") then
        print(tostring(t).." {")
        sub_print_r(t,"  ")
        print("}")
    else
        sub_print_r(t,"  ")
    end
    print()
end

function love.draw()
	local vx,vy = 8,128+32 --map offset
	for y=1,#map do for x=#map[y],1,-1 do local m=map[y][x]
		local tx,ty = ts*(x-1)+ts*(y-1)+vx,-ts/2*x + ts/2*(y-1)+vy
		lg.setColor(255,255,255)
		lg.draw(iTile,tx,ty-16)
		
		-- Draw multiple tiles the amount of times their height is
		local hh
		for hh = 1, map[y][x].h do
		
			lg.draw(iTile,tx,ty-16 - (hh * 11)) -- modify 16 (offset) and 11 (multiplier) to fit tile needs
		
		end
		
			--lg.polygon("fill", tx,ty, tx+ts,ty+ts/2, tx+ts*2,ty, tx+ts,ty-ts/2)
			
		lg.setColor(0,0,0,170)
		lg.print(m.h,tx+28,ty - (map[y][x].h * 11))
	end end
	
	plyr.col:draw("fill") -- this is the players collision but it doubles as a drop shadow
	lg.setColor(255,255,255)
	lg.draw(pAnm[pAnm.f], plyr.x,plyr.y+plyr.z, 0,1,1, plyr.img:getWidth()/2,plyr.img:getHeight()) -- player
	lg.print(plyr.z,0,height-20)
	
	lg.rectangle("line",0,0,400,240)
end

function love.keypressed(k)
	if k=="escape" then love.event.quit() end
end